{
 Version - 1.0 
 Copyright (c) - 1997 by Rick Sanden
 Legal -  We are not responsible for anything at all. This is Freeware.

 E-Mail - SandMan-Inc@worldnet.att.net (mail me if you have bugs thanks)

 Description - This Component will use recusion to search the Win95 registry
               (starting at Enum/ISAPNP) for all Modem class entries. Upon
               finding a Modem class entry the component will store the info
               on that entry for later use.

               Note: This component could be modified slightly to search the
                     registry for anything. Have fun 
 Functions:
  FindModem - This function searches and sets up the info structures.
              Always call him before any other function.
              Returns the number of modems found or a error code.
            
  GetFirst -  Gets the first modem found.
              Returns TModemInfo record.
             
  GetLast -   Gets the last modem found
              Returns TModemInfo record.     
    
  Get -       Gets the modem info on certain modem.
              Input the modem #
              Returns TModemInfo record.

              Example: Get(2) gets the info on the second modem found.
              
  GetError -  Gets the error message on the error returned by findmodem.
              Input the error number.
              Returns a string explaining the error.
                      
 History
 ------------------------------
 May 9, 97 -  Initial Release

}
unit SModem;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Registry, FileCtrl;

Const
  {Hope we do not have more than 10 modems attached}
  MAX_COM = 10;
  DEFAULTDEBUGPATH = '.\debug.txt';
  NOREGENUM = -2;
  NOERROR = 0;
  NOMODEM = 0;

type
  { This structures copies what is stored in the registry
    for a modem. }
  TModemInfo = record
   ModemIsThere: boolean;
   Classtype : string;
   CompatibleIDs: string;
   ConfigFlags: TRegdataInfo;
   DeviceDesc: string;
   Driver: string;
   FriendlyName: string;
   HardwareID: string;
   Mfg: string;
   PORTNAME: string;
  end;
   
  EOutOfRange = class(EListError);
  TSenseModem = class(TComponent)
  private
    {This array kepts all info on modems}
    FModemArray: array[1..MAX_COM] of TModemInfo;
    FDebugFile: Boolean;
    FDebugFilePath: string;
    FError: Integer;
    FModemCount: Integer; 
    procedure CheckKeyForModem; 
    procedure SetEmpty; 
    
  protected
    procedure SetDebugPath(path:string); 
  public
    constructor Create( AOwner: TComponent); override;
    destructor Destroy; override;
    function FindModem: Integer;
    function GetFirst: TModemInfo;
    function GetLast: TModemInfo;
    function Get(modem_number:integer): TModemInfo;
    function GetError: String;
    
  published
    property DebugFile : boolean read FDebugFile write FDebugFIle default False;          
    property DebugFilePath : string read FDebugFilePath write SetDebugPath;          
    function InternalFindModem(StartLocation: string): Integer;
  end;

procedure Register;

Var
 Debug_file:TextFile;
 RegistryEntries: TRegistry;
 

implementation

{ Default constructor 
  Initial variable ssetup  }
constructor TSenseModem.Create( AOwner: Tcomponent);
begin
   inherited Create( AOwner);
   DebugFile := False;
   DebugFilePath := FDebugFilePath;
   FError := NOERROR;
   FModemCount := 0;
   RegistryEntries :=  TRegistry.Create;
    
end;

{ Default Destructor }
destructor TSenseModem.Destroy;
begin
  RegistryEntries.Free;
end;

{ Set all info structure to intial empty
  state}
procedure TSenseModem.SetEmpty;
Var i: integer;
begin
  for i:=1 to MAX_COM do
  begin
    FModemArray[i].ModemIsThere := False;
    FModemArray[i].Classtype := '';
    FModemArray[i].CompatibleIDs := '';
    FModemArray[i].DeviceDesc := '';
    FModemArray[i].Driver := '';
    FModemArray[i].FriendlyName := '';
    FModemArray[i].HardwareID := '';
    FModemArray[i].Mfg := '';
    FModemArray[i].PORTNAME := '';
  end;
end; 

{Check to see if the debug file path exists.
 If it does not then setup the default path}
procedure TSenseModem.SetDebugPath(path:string); 
begin
  if (DirectoryExists(ExtractFilePath(path))) then
      FDebugFilePath := path
  else
    FDebugFilePath := DEFAULTDEBUGPATH;
end;

{Get info on requested modem}
function TSenseModem.Get(modem_number:integer): TModemInfo;
begin
  { Error out if number requested is > max }
  { Note: We always return a info structure even if it is
          empty. The ModemIsThere field should be used by
          the user to double check to see if there is real
          info.} 
  if (modem_number > MAX_COM) then
   raise EOutOfRange.CreateFmt('Input into "Get" function out of range',[modem_number])
  else
   Get := FModemArray[modem_number]; 
end;

{ Get info on the first modem found }
function TSenseModem.GetFirst: TModemInfo;
begin
  GetFirst := FModemArray[1];
end;

{ Get info on last modem found }
function TSenseModem.GetLast: TModemInfo;
begin
  GetLast := FModemArray[FModemCount];
end;

{Get string explanation of error code returned
  by findmodem}
function TSenseModem.GetError: String;
begin
   case FError of
    NOERROR: GetError := '';
    NOREGENUM: GetError:= 'Registry key ENUM could not be opened.';
   end;
end;

function TSenseModem.FindModem:integer;
begin

  { Set stuff to inital }
  { Yes this may be have already been done in the constructor }
  FError := NOERROR; 
  FModemCount := 0;
  
  { Set Root key to HKEY_LOCAL_MACHINE }
  RegistryEntries.RootKey := HKEY_LOCAL_MACHINE;  
  
  { Open debug file if it exists}
  if (DebugFile) then 
  begin
     AssignFile(Debug_file,DebugFilePath);
     Rewrite(Debug_file);
  end;
  
  { Set everyone to false}
  SetEmpty;
  
  {Modem should be in Enum/ISAPNP}   
  if (RegistryEntries.keyExists('Enum')) then
    RegistryEntries.OpenKey('Enum',False)
  else
  begin
    {This tells us that for somereason we could not
     open the registry}
    FindModem := NOREGENUM;
    FError := NOREGENUM;
    exit;  
  end;   

  { Find them }   
  InternalFindModem('ISAPNP');
  { Return count }
  FindModem :=  FModemCount; 
   
  { Close Debug file if it exists} 
  if (DebugFile) then 
     CloseFile(Debug_file);

  RegistryEntries.CloseKey;   

end;

procedure TSenseModem.CheckKeyForModem;
begin
  { Look for Class Value in current key }
  if (registryEntries.ValueExists('Class') and
      (FModemCount < MAX_COM) ) then
  begin
     { Check for Modem class }
     if (registryEntries.ReadString('Class') = 'Modem') then
     begin
       FModemCount := FModemCount + 1; 
       FModemArray[FModemCount].ModemIsThere := True;
       {Start Filling Modem Structure }
       FModemArray[FModemCount].Classtype := registryEntries.ReadString('Class'); 
        
       if (registryEntries.ValueExists('CompatibleIDs') and 
           (registryEntries.GetDataType('CompatibleIDs') = rdString) )  then
           FModemArray[FModemCount].CompatibleIDs :=
                (registryEntries.ReadString('CompatibleIDs'))
       else
           FModemArray[FModemCount].CompatibleIDs := ' ';  

       if (registryEntries.ValueExists('DeviceDesc') and 
           (registryEntries.GetDataType('DeviceDesc') = rdString) )  then
           FModemArray[FModemCount].DeviceDesc :=
                (registryEntries.ReadString('DeviceDesc'))
       else
           FModemArray[FModemCount].DeviceDesc := ' ';

      if (registryEntries.ValueExists('Driver') and 
           (registryEntries.GetDataType('Driver') = rdString) )  then
           FModemArray[FModemCount].Driver :=
                (registryEntries.ReadString('Driver'))
       else
           FModemArray[FModemCount].Driver := ' ';

      if (registryEntries.ValueExists('FriendlyName') and 
           (registryEntries.GetDataType('FriendlyName') = rdString) )  then
           FModemArray[FModemCount].FriendlyName :=
                (registryEntries.ReadString('FriendlyName'))
       else
           FModemArray[FModemCount].FriendlyName := ' ';

      if (registryEntries.ValueExists('HardwareID') and 
           (registryEntries.GetDataType('HardwareID') = rdString) )  then
           FModemArray[FModemCount].HardwareID :=
                (registryEntries.ReadString('HardwareID'))
       else
           FModemArray[FModemCount].HardwareID := ' ';

      if (registryEntries.ValueExists('Mfg') and 
           (registryEntries.GetDataType('Mfg') = rdString) )  then
           FModemArray[FModemCount].Mfg :=
                (registryEntries.ReadString('Mfg'))
       else
           FModemArray[FModemCount].Mfg := ' ';

      if (registryEntries.ValueExists('PORTNAME') and 
           (registryEntries.GetDataType('PORTNAME') = rdString) )  then
           FModemArray[FModemCount].PORTNAME :=
                (registryEntries.ReadString('PORTNAME'))
       else
           FModemArray[FModemCount].PORTNAME := ' ';
      
      if (registryEntries.ValueExists('ConfigFlags')) then
           registryEntries.GetDataInfo('ConfigFlags',
                  FModemArray[FModemCount].ConfigFlags);
  
     end;   
    
     if (FDebugFile) then
        writeln(Debug_file,'Found a Match -> ' + 
        FModemArray[FModemCount].FriendlyName);  
  end;

end; 

{This function does all the work. It uses recusion to walk
 through the tree structure,of the registry looking, for Modem
 Classes.} 
function TSenseModem.InternalFindModem(StartLocation: string):integer;
var i,j,k: integer;
    Alist: TstringList;
    Alist2:TstringList;
    Valuelist: TstringList;
    currentPath:String;
    myPath :String;
begin

  { Create all }
  Alist := TstringList.Create;
  Alist2 := TstringList.Create;
  ValueList := TstringList.Create;

  
  {Open key at current position}
  RegistryEntries.OpenKey(StartLocation,False);
  {Get the current path in registry }
  myPath := registryEntries.currentPath;
  
  if (FDebugFile) then
  begin
     writeln(Debug_file,'* Starting FindModem with Key ' + StartLocation);
     writeln(Debug_file,'* Current Path ' + myPath);
  end;

  { Get the value list for this key }
  RegistryEntries.GetValueNames(ValueList);
  { Look for "Modem" class in value list}
  CheckKeyForModem;
  { Now look for sub keys}
  if (RegistryEntries.HasSubKeys) then
  begin
     { Get a list of sub keys}
     RegistryEntries.GetKeyNames(Alist);
     { Walk through sub keys looking for Modem classes }
     for  i:= 0 to (AList.Count-1) do
     begin
       if (FDebugFile) then
          writeln(Debug_file,'  -> Calling FindModem with ' + AList.Strings[i] +
                  ' Count ' + Inttostr(i));
       { Call myself to check subkeys }
       { This is the neat call }           
       InternalFindModem(AList.Strings[i]);
       { Back up the tree to original path }
       RegistryEntries.CloseKey();
       RegistryEntries.OpenKey(myPath,False);
     end;  
  end     
  else { Need to check first key for modem class}
    CheckKeyForModem;

  { Free All } 
  Alist.Free;
  Alist2.Free;
  ValueList.Free;
end;

procedure Register;
begin
  RegisterComponents('System', [TSenseModem]);
end;

end.
 
